@@ -148,8 +148,6 @@ class LsystemWidget(PGLWidget):
     _view_module = Unicode(module_name).tag(sync=True)
     _view_module_version = Unicode(module_version).tag(sync=True)
 
-    __visualparameters = []
-    __scalars = []
     __trees = []
     __filename = ''
     __do_abort = False
@@ -165,17 +163,31 @@ class LsystemWidget(PGLWidget):
     }).tag(sync=True, to_json=scene_to_json)
     animate = Bool(False).tag(sync=True)
     dump = Unicode('').tag(sync=False)
+    is_magic = Bool(False).tag(sync=True)
+
     __editor = None
     __codes = []
     __derivationStep = 0
     __lsystem = None
+    __extra_context = {}
 
-    def __init__(self, filename, options={}, unit=Unit.m, animate=False, dump='', **kwargs):
-        self.__filename = filename if filename.endswith('.lpy') else filename + '.lpy'
+    def __init__(self, filename, code='', unit=Unit.m, animate=False, dump='', context={}, **kwargs):
+
+        self.__filename = filename if filename and filename.endswith('.lpy') else str(filename) + '.lpy'
         self.__lsystem = lpy.Lsystem()
-        with io.open(self.__filename, 'r') as file:
-            self.__codes = file.read().split(lpy.LpyParsing.InitialisationBeginTag)
-            self.__codes.insert(1, f'\n{lpy.LpyParsing.InitialisationBeginTag}\n')
+        self.__extra_context = context
+        code_ = ''
+        if Path(self.__filename).is_file():
+            with io.open(self.__filename, 'r') as file:
+                code_ = file.read()
+        else:
+            self.__filename = ''
+            self.is_magic = True
+            code_ = code
+
+        self.__codes = code_.split(lpy.LpyParsing.InitialisationBeginTag)
+        self.__codes.insert(1, f'\n{lpy.LpyParsing.InitialisationBeginTag}\n')
+
         if len(self.__codes) < 2:
             raise ValueError('No L-Py code found')
         # if a json file with the same name is present load context from the json file
@@ -208,7 +220,10 @@ class LsystemWidget(PGLWidget):
 
     def __initialize_lsystem(self):
         self.__lsystem.filename = self.__filename
-        self.__lsystem.set(''.join(self.__codes), {})
+        self.__lsystem.set(''.join(self.__codes), self.__extra_context)
+        # context = self.__lsystem.context()
+        # for key, value in self.__extra_context.items():
+        #     context[key] = value
         self.derivationLength = self.__lsystem.derivationLength
         self.__trees = []
         self.__trees.append(self.__lsystem.axiom)
@@ -227,29 +242,51 @@ class LsystemWidget(PGLWidget):
 
     def __initialisation_function(self, context_obj):
         def build_context(context_obj, context):
-            for key, value in context_obj.items():
-                if isinstance(value, dict):
-                    if key == 'scalars':
-                        for name, scalar in value.items():
-                            context[name] = scalar['value']
-                    elif key == 'materials':
-                        for name, material in value.items():
-                            m = dict(material)
-                            index = m.pop('index')
-                            context[name] = pgl.Material(**m)
-                            context.turtle.setMaterial(index, context[name])
-                    elif key == 'functions':
-                        for name, function in value.items():
-                            if 'NurbsCurve2D' in function:
-                                points = function['NurbsCurve2D']
-                                context[name] = pgl.QuantisedFunction(
-                                    pgl.NurbsCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in points]))
-                                )
-                    elif key == 'curves':
-                        for name, curve in value.items():
-                            if 'NurbsCurve2D' in curve:
-                                points = curve['NurbsCurve2D']
-                                context[name] = pgl.NurbsCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in points]))
+            for material in context_obj['materials']:
+                material = dict(material)
+                index = material.pop('index')
+                name = material.pop('name')
+                context[name] = pgl.Material(**material)
+                context.turtle.setMaterial(index, context[name])
+            for catergory in context_obj['parameters']:
+                if catergory['enabled']:
+                    for scalar in catergory['scalars']:
+                        context[scalar['name']] = scalar['value']
+                    for curve in catergory['curves']:
+                        if curve['is_function']:
+                            context[curve['name']] = pgl.QuantisedFunction(
+                                pgl.NurbsCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in curve['points']]))
+                            )
+                        elif curve['type'] == 'NurbsCurve2D':
+                            context[curve['name']] = pgl.NurbsCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in curve['points']]))
+                        elif curve['type'] == 'BezierCurve2D':
+                            context[curve['name']] = pgl.BezierCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in curve['points']]))
+                        elif curve['type'] == 'Polyline2D':
+                            context[curve['name']] = pgl.Polyline2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in curve['points']]))
+
+            # for key, value in context_obj.items():
+            #     if isinstance(value, dict):
+            #         if key == 'scalars':
+            #             for name, scalar in value.items():
+            #                 context[name] = scalar['value']
+            #         elif key == 'materials':
+            #             for name, material in value.items():
+            #                 m = dict(material)
+            #                 index = m.pop('index')
+            #                 context[name] = pgl.Material(**m)
+            #                 context.turtle.setMaterial(index, context[name])
+            #         elif key == 'functions':
+            #             for name, function in value.items():
+            #                 if 'NurbsCurve2D' in function:
+            #                     points = function['NurbsCurve2D']
+            #                     context[name] = pgl.QuantisedFunction(
+            #                         pgl.NurbsCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in points]))
+            #                     )
+            #         elif key == 'curves':
+            #             for name, curve in value.items():
+            #                 if 'NurbsCurve2D' in curve:
+            #                     points = curve['NurbsCurve2D']
+            #                     context[name] = pgl.NurbsCurve2D(pgl.Point3Array([pgl.Vector3(p[0], p[1], 1) for p in points]))
 
         codes = [
             '\n__lpy_code_version__ = 1.1',
@@ -283,17 +320,22 @@ class LsystemWidget(PGLWidget):
     def __rewind(self):
         self.__lsystem.clear()
         self.__derivationStep = 0
-        with io.open(self.__filename, 'r') as file:
-            self.__codes = file.read().split(lpy.LpyParsing.InitialisationBeginTag)
-            self.__codes.insert(1, f'\n{lpy.LpyParsing.InitialisationBeginTag}\n')
-        if len(self.__codes) < 2:
-            raise ValueError('No L_Py code found')
-        if self.__editor is not None:
-            self.__on_lpy_context_change(self.__editor.lpy_context)
-        elif Path(self.__filename[0:-3] + 'json').is_file():
-            self.__editor = ParameterEditor(self.__filename[0:-3] + 'json')
-            self.__editor.on_lpy_context_change = self.__on_lpy_context_change
-            self.__on_lpy_context_change(self.__editor.lpy_context)
+
+        if self.__filename:
+            with io.open(self.__filename, 'r') as file:
+                self.__codes = file.read().split(lpy.LpyParsing.InitialisationBeginTag)
+                self.__codes.insert(1, f'\n{lpy.LpyParsing.InitialisationBeginTag}\n')
+            if len(self.__codes) < 2:
+                raise ValueError('No L_Py code found')
+            if self.__editor is not None:
+                self.__on_lpy_context_change(self.__editor.lpy_context)
+            elif Path(self.__filename[0:-3] + 'json').is_file():
+                self.__editor = ParameterEditor(self.__filename[0:-3] + 'json')
+                self.__editor.on_lpy_context_change = self.__on_lpy_context_change
+                self.__on_lpy_context_change(self.__editor.lpy_context)
+            else:
+                self.__initialize_lsystem()
+                self.__set_scene(0)
         else:
             self.__initialize_lsystem()
             self.__set_scene(0)
