@@ -178,16 +178,13 @@ self.onmessage = function (evt) {
             postMessage({ error: err.toString() });
         }
     } else {
-        if ('wasmURL' in data) {
+        const { wasmBinary } = data;
+        if (wasmBinary) {
             try {
-                fetch(data.wasmURL)
-                    .then(res => res.arrayBuffer())
-                    .then(wasmBinary => {
-                        PGL({ wasmBinary })
-                            .then(pgl_ => {
-                                pgl = pgl_;
-                                self.postMessage({ initialized: true });
-                            });
+                PGL({ wasmBinary })
+                    .then(pgl_ => {
+                        pgl = pgl_;
+                        self.postMessage({ initialized: true });
                     });
             } catch (err) {
                 console.log(err)
